<?php
// This file is part of Moodle - https://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.

/**
 * This file contains the library functions for the AAT plugin.
 * @package     local_aatplugin
 * @copyright   2023 Joseph Larochelle <larochellejo@hotmail.com>
 * @license     https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */


/**
 * Insert a link to index.php on the site front page navigation menu if the user has the
 * View course overview report capability.
 *
 * @param navigation_node $frontpage Node representing the front page in the navigation tree.
 */
function local_aatplugin_extend_navigation_frontpage(navigation_node $frontpage) {
    require_login();
    $context = context_system::instance();
    if (has_capability('report/courseoverview:view', $context)) {
        $frontpage->add(
                get_string('pluginname', 'local_aatplugin'),
                new moodle_url('/local/aatplugin/index.php'),
                navigation_node::TYPE_CUSTOM
        );
    }
}
/**
 * Insert a link to index.php on the course page navigation menu if the user has the
 * View course overview report capability.
 *
 * @param navigation_node $course Node representing the front page in the navigation tree.
 */
function local_aatplugin_extend_navigation_course(navigation_node $course) {
    require_login();
    global $COURSE;
    $courseid = $COURSE->id;
    $context = context_course::instance($courseid);
    if (has_capability('report/courseoverview:view', $context)) {
        $course->add(
                get_string('pluginname', 'local_aatplugin'),
                new moodle_url('/local/aatplugin/index.php'),
                navigation_node::TYPE_CUSTOM
        );
    }
}
/**
 * Combines all 'install.xml' files from a given directory and its subdirectories into one large XML file.
 *
 * @param string $directory The base directory to start searching for 'install.xml' files.
 */
function local_aatplugin_combinexmlfiles(string $directory) {
    require_once('../../config.php');
    require_login();
    global $CFG;
    $dom = new DOMDocument('1.0', 'UTF-8');
    $root = $dom->createElement('XMLDB');
    $dom->appendChild($root);

    $tableselement = $dom->createElement('TABLES');
    $root->appendChild($tableselement);

    // Initialize a RecursiveIteratorIterator to iterate through directories and files.
    $iterator = new RecursiveIteratorIterator(
            new RecursiveDirectoryIterator($directory),
            RecursiveIteratorIterator::SELF_FIRST
    );

    // Iterate over each item in the directory and its subdirectories.
    foreach ($iterator as $file) {
        if ($file->isFile() && $file->getFilename() == 'install.xml') {
            // Load the XML content of the 'install.xml' file.
            $xml = simplexml_load_file($file->getPathname());
            // Iterate over each '<TABLE>' element within the file.
            foreach ($xml->TABLES->TABLE as $table) {
                $domtable = dom_import_simplexml($table);
                $import = $dom->importNode($domtable, true);
                $tableselement->appendChild($import);
            }
        }
    }
    // Save the combined XML document to a file in Moodle's data root under 'aatplugin/combined_XML_files.xml'.
    $dom->formatOutput = true;
    $dom->save($CFG->dataroot. '/aatplugin/combined_XML_files.xml');
}
/**
 * Handles the form submission, generating an SQL dump if necessary.
 *
 * @param dump_form $dumpform The form instance.
 */
function local_aatplugin_process_form_submission(\local_aatplugin\form\dump_form $dumpform ) {
    if ($data = $dumpform->get_data()) {
        $selectedcourses = optional_param_array('courseid', [], PARAM_INT);
        if (empty($selectedcourses)) {
            display_selection_error($dumpform);
            return;
        }
        local_aatplugin_generate_and_download_sql_dump($selectedcourses);
    } else {
        echo '<br><br>';
        $dumpform->display();
    }
}
/**
 * Displays an error message when no courses are selected and re-displays the form.
 *
 * @param dump_form $dumpform The form instance.
 */
function local_aatplugin_display_selection_error(\local_aatplugin\form\dump_form $dumpform) {
    echo '<br>';
    echo html_writer::tag('h5', "You have not selected any courses, please select at least one course");
    echo '<br>';
    $dumpform->display();
}
/**
 * Displays the link to the AAT website.
 */
function local_aatplugin_display_instructions() {
    echo '<h5>Instructions:</h5><br>';
    // Create a list of instructions.
    echo html_writer::tag('p', "1. Select the courses you want to export and click the 'Generate SQL Dump' button");
    echo html_writer::tag('p', "2. Once the file is generated, click the 'Download SQL Dump' link to download the file
    to your computer. Take note of the file location.");
    echo html_writer::tag('p', "3. Head over to the AAT website (link provided below) and log in using your credentials");
    echo html_writer::tag('p', "4. Within the AAT website, navigate to the 'Data Source' tab and click on the 'Upload' button");
    echo html_writer::tag('p', "5. Select 'Postgres' as the vendor, type in a name for the database, and select the file that was
                           downloaded in step 2. Click 'Save' to complete the process.");
    echo html_writer::tag('p', "You now should have access to the data within the AAT tool.");
    echo html_writer::tag('a', "https://www.academicanalytics.ca/", [
            'href' => 'https://www.academicanalytics.ca/',
            'target' => '_blank' ,
    ]);
}
/**
 * Generates the SQL dump for selected courses and provides a download link.
 *
 * @param array $selectedcourses The IDs of the selected courses.
 */
function local_aatplugin_generate_and_download_sql_dump(array $selectedcourses) {
    global $CFG;
    global $DB;
    $datadump = new local_aatplugin\data_dump($DB, $CFG);
    $courseidsstring = implode('_', $selectedcourses);
    $filename = "course_{$courseidsstring}_dump_plugin.sql";
    $filepath = $CFG->dataroot . "/" . $filename;
    $datadump->generate_sql_dump($selectedcourses, $filepath);

    // Display the download link.
    echo '<br>';
    echo html_writer::tag('h4', "Your file was created, click the link below to download it");
    echo '<br>';
    echo html_writer::start_tag('div', ['style' => 'display: flex; justify-content: center; align-items: center;']);
    echo html_writer::tag('a', 'Download SQL Dump', [
            'href' => new moodle_url('/local/aatplugin/downloadfile.php', ['file' => $filepath]),
            'class' => 'btn btn-primary' ,
    ]);
    echo html_writer::end_tag('div');

    // Add a link to go back to the course selection page.
    echo '<br>';
    echo html_writer::start_tag('div', ['style' => 'display: flex; justify-content: center; align-items: center;']);
    echo html_writer::tag('a', 'Go back to course selection', [
            'href' => new moodle_url('/local/aatplugin/index.php'),
            'class' => 'btn btn-secondary' ,
    ]);
    echo html_writer::end_tag('div');
}
/**
 * Fetches the courses where the current user is has the 'report/courseoverview:view' capability.
 *
 * @return array An array of course objects.
 */
function local_aatplugin_fetch_user_courses(): array {
    global $USER;

    // Specify the capability you want to check.
    $capability = 'report/courseoverview:view';

    // Get the list of courses where the user is enrolled.
    $courses = enrol_get_users_courses($USER->id, true);

    // Prepare an array to hold the course ids and fullnames.
    $coursedetails = [];

    // Loop through the courses and check if the user has the specified capability.
    foreach ($courses as $course) {
        $context = context_course::instance($course->id);
        if (has_capability($capability, $context, $USER->id)) {
            $coursedetails[] = ['id' => $course->id, 'fullname' => $course->fullname];
        }
    }
    return $coursedetails;
}
